@@ -1,5 +1,5 @@
 /***************************************************************************
- * CVSID: $Id: nm-vpn-properties.c,v 1.8 2005/10/19 16:08:49 rml Exp $
+ * CVSID: $Id: nm-vpn-properties.c,v 1.13 2006/02/27 18:16:48 rml Exp $
  *
  * nm-vpn-properties.c : GNOME UI dialogs for manipulating VPN connections
  *
@@ -48,7 +48,6 @@
 
 static GladeXML *xml;
 static GConfClient *gconf_client;
-
 static GtkWidget *dialog;
 static GtkWindow *druid_window;
 static GtkTreeView *vpn_conn_view;
@@ -61,7 +60,6 @@ static GnomeDruidPageEdge *druid_confirm_page;
 static GtkComboBox *vpn_type_combo_box;
 static GtkVBox *vpn_type_details;
 static GtkDialog *edit_dialog;
-
 static GSList *vpn_types;
 
 static NetworkManagerVpnUI *
@@ -152,16 +150,19 @@ add_vpn_connection (const char *conn_name, const char *service_name, GSList *con
 
 	/* routes */
 	gconf_key = g_strdup_printf ("%s/routes", conn_gconf_path);
+
+#if 0
 	{
-/*
 		GSList *i;
 
 		i = NULL;
 		i = g_slist_append (i, "172.16.0.0/16");
-*/
 		gconf_client_set_list (gconf_client, gconf_key, GCONF_VALUE_STRING, routes, NULL);
-		/*g_slist_free (i);*/
+		g_slist_free (i);
 	}
+#else
+		gconf_client_set_list (gconf_client, gconf_key, GCONF_VALUE_STRING, routes, NULL);
+#endif
 
 	gconf_client_suggest_sync (gconf_client, NULL);
 
@@ -265,14 +266,7 @@ static void vpn_druid_vpn_details_page_prepare (GnomeDruidPage *druidpage,
 						GtkWidget *widget,
 						gpointer user_data)
 {
-	/*printf ("vpn_details_prepare!\n");*/
-
-	gnome_druid_set_buttons_sensitive (druid, 
-					   TRUE,
-					   FALSE,
-					   TRUE,
-					   FALSE);	
-
+	gnome_druid_set_buttons_sensitive (druid, TRUE, FALSE, TRUE, FALSE);	
 }
 
 static gboolean vpn_druid_vpn_details_page_next (GnomeDruidPage *druidpage,
@@ -288,9 +282,8 @@ static gboolean vpn_druid_vpn_details_page_next (GnomeDruidPage *druidpage,
 
 	/* validate input */
 	vpn_ui = (NetworkManagerVpnUI *) g_slist_nth_data (vpn_types, gtk_combo_box_get_active (vpn_type_combo_box));
-	if (vpn_ui != NULL) {
+	if (vpn_ui != NULL)
 		is_valid = vpn_ui->is_valid (vpn_ui);
-	}
 
 	return !is_valid;
 }
@@ -339,11 +332,8 @@ static gboolean vpn_druid_vpn_confirm_page_finish (GnomeDruidPage *druidpage,
 	return FALSE;
 }
 
-static gboolean vpn_druid_cancel (GnomeDruid *druid,
-				  gpointer user_data)
+static gboolean vpn_druid_cancel (GnomeDruid *ignored_druid, gpointer user_data)
 {
-	/*printf ("vpn_druid_cancel!\n");*/
-
 	gtk_widget_hide_all (GTK_WIDGET (druid_window));
 	return FALSE;
 }
@@ -357,25 +347,22 @@ add_cb (GtkButton *button, gpointer user_data)
 	GList *i;
 	GList *children;
 
-	/*printf ("add_cb\n");*/
-
 	/* Bail out if we don't have any VPN implementations on our system */
 	if (vpn_types == NULL || g_slist_length (vpn_types) == 0) {
-		GtkWidget *dialog;
+		GtkWidget *err_dialog;
 
-		dialog = gtk_message_dialog_new (NULL,
+		err_dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
 						 _("Cannot add VPN connection"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dialog),
 		   _("No suitable VPN software was found on your system. Contact your system administrator."));
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
+		gtk_dialog_run (GTK_DIALOG (err_dialog));
+		gtk_widget_destroy (err_dialog);
 		goto out;
 	}
 
-
 	/* remove existing VPN widget */
 	children = gtk_container_get_children (GTK_CONTAINER (vpn_type_details));
 	for (i = children; i != NULL; i = g_list_next (i)) {
@@ -413,20 +400,20 @@ import_settings (const char *svc_name, const char *name)
 	/* Bail out if we don't have the requested VPN implementation on our system */
 	if (vpn_ui == NULL) {
 		char *basename;
-		GtkWidget *dialog;
+		GtkWidget *err_dialog;
 
 		basename = g_path_get_basename (name);
 
-		dialog = gtk_message_dialog_new (NULL,
+		err_dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
 						 _("Cannot import VPN connection"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dialog),
 							  _("Cannot find suitable software for VPN connection type '%s' to import the file '%s'. Contact your system administrator."),
 							  svc_name, basename);
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
+		gtk_dialog_run (GTK_DIALOG (err_dialog));
+		gtk_widget_destroy (err_dialog);
 		g_free (basename);
 		goto out;
 	}
@@ -443,7 +430,6 @@ import_settings (const char *svc_name, const char *name)
 	w = glade_xml_get_widget (xml, "vpn-druid-vpn-details-page");
 	gnome_druid_set_page (druid, GNOME_DRUID_PAGE (w));
 
-
 	/* show appropriate child */
 	w = vpn_ui->get_widget (vpn_ui, NULL, NULL, NULL);
 	if (w != NULL) {	
@@ -454,9 +440,9 @@ import_settings (const char *svc_name, const char *name)
 			gtk_container_remove (GTK_CONTAINER (old_parent), w);
 		gtk_container_add (GTK_CONTAINER (vpn_type_details), w);
 		gtk_widget_unref (w);
-		
 		gtk_widget_show_all (w);
-	}	
+	}
+
 	vpn_ui->set_validity_changed_callback (vpn_ui, vpn_druid_vpn_validity_changed, NULL);
 
 	vpn_ui->import_file (vpn_ui, name);
@@ -465,6 +451,7 @@ import_settings (const char *svc_name, const char *name)
 
 	gtk_window_set_policy (druid_window, FALSE, FALSE, TRUE);
 	gtk_widget_show_all (GTK_WIDGET (druid_window));
+
 out:
 	;
 }
@@ -557,19 +544,19 @@ retrieve_data_from_selected_connection (NetworkManagerVpnUI **vpn_ui,
 
 	*vpn_ui = find_vpn_ui_by_service_name (conn_service_name);
 	if (*vpn_ui == NULL) {
-		GtkWidget *dialog;
+		GtkWidget *err_dialog;
 
-		dialog = gtk_message_dialog_new (NULL,
+		err_dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
 						 _("Error retrieving VPN connection '%s'"),
 						 *conn_name);
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-  _("Could not find the UI files for VPN connection type '%s'. Contact your system administrator."),
-							  conn_service_name);
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dialog),
+		    _("Could not find the UI files for VPN connection type '%s'. Contact your system administrator."),
+		    conn_service_name);
+		gtk_dialog_run (GTK_DIALOG (err_dialog));
+		gtk_widget_destroy (err_dialog);
 		goto out;
 	}
 
@@ -581,9 +568,9 @@ retrieve_data_from_selected_connection (NetworkManagerVpnUI **vpn_ui,
 
 	*conn_vpn_data = NULL;
 	for (i = conn_vpn_data_gconfvalue; i != NULL; i = g_slist_next (i)) {
-		const char *value;
-		value = gconf_value_get_string ((GConfValue *) i->data);
-		*conn_vpn_data = g_slist_append (*conn_vpn_data, (gpointer) value);
+		const char *val;
+		val = gconf_value_get_string ((GConfValue *) i->data);
+		*conn_vpn_data = g_slist_append (*conn_vpn_data, (gpointer) val);
 	}
 
 
@@ -596,9 +583,9 @@ retrieve_data_from_selected_connection (NetworkManagerVpnUI **vpn_ui,
 	conn_routes_gconfvalue = gconf_value_get_list (value);
 	*conn_routes = NULL;
 	for (i = conn_routes_gconfvalue; i != NULL; i = g_slist_next (i)) {
-		const char *value;
-		value = gconf_value_get_string ((GConfValue *) i->data);
-		*conn_routes = g_slist_append (*conn_routes, (gpointer) value);
+		const char *val;
+		val = gconf_value_get_string ((GConfValue *) i->data);
+		*conn_routes = g_slist_append (*conn_routes, (gpointer) val);
 	}
 
 	result = TRUE;
@@ -654,7 +641,6 @@ edit_cb (GtkButton *button, gpointer user_data)
 		new_conn_routes = vpn_ui->get_routes (vpn_ui);
 
 		if (strcmp (new_conn_name, conn_name) == 0) {
-
 			/* same name, just update properties and routes */
 			g_snprintf (key, sizeof (key), "%s/vpn_data", conn_gconf_path);
 			gconf_client_set_list (gconf_client, key, GCONF_VALUE_STRING, new_conn_data, NULL);
@@ -663,7 +649,6 @@ edit_cb (GtkButton *button, gpointer user_data)
 
 			gconf_client_suggest_sync (gconf_client, NULL);
 		} else {
-
 			/* remove old entry */
 			g_snprintf (key, sizeof (key), "%s/name", conn_gconf_path);
 			gconf_client_unset (gconf_client, key, NULL);
@@ -693,10 +678,9 @@ edit_cb (GtkButton *button, gpointer user_data)
 			g_slist_foreach (new_conn_routes, (GFunc)g_free, NULL);
 			g_slist_free (new_conn_routes);
 		}
-
-
 	}
 
+	gtk_widget_hide (GTK_WIDGET (vpn_edit_widget));
 	gtk_widget_hide (GTK_WIDGET (edit_dialog));
 
 out:
@@ -710,7 +694,7 @@ delete_cb (GtkButton *button, gpointer user_data)
 	GtkTreeSelection *selection;
 	gchar *conn_gconf_path;
 	gchar *conn_name;
-	GtkWidget *dialog;
+	GtkWidget *confirm_dialog;
 	int response;
 
 	/*printf ("delete\n");*/
@@ -722,16 +706,16 @@ delete_cb (GtkButton *button, gpointer user_data)
 		goto out;
 
 	gtk_tree_model_get (GTK_TREE_MODEL (vpn_conn_list), &iter, VPNCONN_NAME_COLUMN, &conn_name, -1);
-	dialog = gtk_message_dialog_new (NULL,
+	confirm_dialog = gtk_message_dialog_new (NULL,
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_WARNING,
 					 GTK_BUTTONS_CANCEL,
 					 _("Delete VPN connection \"%s\"?"), conn_name);
-	gtk_dialog_add_buttons (GTK_DIALOG (dialog), GTK_STOCK_DELETE, GTK_RESPONSE_OK, NULL);
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+	gtk_dialog_add_buttons (GTK_DIALOG (confirm_dialog), GTK_STOCK_DELETE, GTK_RESPONSE_OK, NULL);
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (confirm_dialog),
 						  _("All information about the VPN connection \"%s\" will be lost and you may need your system administrator to provide information to create a new connection."), conn_name);
-	response = gtk_dialog_run (GTK_DIALOG (dialog));
-	gtk_widget_destroy (dialog);
+	response = gtk_dialog_run (GTK_DIALOG (confirm_dialog));
+	gtk_widget_destroy (confirm_dialog);
 
 	if (response != GTK_RESPONSE_OK)
 		goto out;
@@ -752,6 +736,9 @@ delete_cb (GtkButton *button, gpointer user_data)
 		/* TODO: remove user_name */
 		g_snprintf (key, sizeof (key), "%s/user_name", conn_gconf_path);
 		gconf_client_unset (gconf_client, key, NULL);
+		g_snprintf (key, sizeof (key), "%s/last_attempt_success", conn_gconf_path);
+		gconf_client_unset (gconf_client, key, NULL);
+
 		gconf_client_unset (gconf_client, conn_gconf_path, NULL);
 
 		gconf_client_suggest_sync (gconf_client, NULL);
@@ -838,7 +825,7 @@ static void get_all_vpn_connections (void)
 				    VPNCONN_USER_CAN_EDIT_COLUMN, conn_user_can_edit,
 				    -1);
 
-		/*
+#if 0
 		printf ("conn_name = '%s'\n", conn_name);
 		printf ("conn_service_name = '%s'\n", conn_service_name);
 		printf ("conn_vpn_data = {");
@@ -851,7 +838,7 @@ static void get_all_vpn_connections (void)
 			}
 			printf ("}\n");
 		}
-		*/
+#endif
 
 error:
 		g_free (vpn_conn->data);
@@ -872,7 +859,7 @@ vpn_list_cursor_changed_cb (GtkTreeView *treeview,
 extern NetworkManagerVpnUI* vpn_ui_factory_dummy (void);
 
 static void
-load_properties_module (GSList **vpn_types, const char *path)
+load_properties_module (GSList **vpn_types_list, const char *path)
 {
 	GModule *module;
 	NetworkManagerVpnUI* (*nm_vpn_properties_factory) (void) = NULL;
@@ -886,7 +873,6 @@ load_properties_module (GSList **vpn_types, const char *path)
 
 	if (!g_module_symbol (module, "nm_vpn_properties_factory", 
 			      (gpointer) &nm_vpn_properties_factory)) {
-		
 		g_warning ("Cannot locate function 'nm_vpn_properties_factory' in '%s': %s", 
 			   path, g_module_error ());
 		g_module_close (module);
@@ -900,13 +886,13 @@ load_properties_module (GSList **vpn_types, const char *path)
 		goto out;
 	}
 
-	*vpn_types = g_slist_append (*vpn_types, impl);
+	*vpn_types_list = g_slist_append (*vpn_types_list, impl);
 
 out:
 	;
 }
 
-#define VPN_NAME_FILES_DIR "/etc/NetworkManager/VPN"
+#define VPN_NAME_FILES_DIR SYSCONFDIR"/NetworkManager/VPN"
 
 static gboolean
 init_app (void)
@@ -929,17 +915,17 @@ init_app (void)
 	xml = glade_xml_new (glade_file, NULL, NULL);
 	g_free (glade_file);
 	if (!xml) {
-		GtkWidget *dialog;
+		GtkWidget *err_dialog;
 
-		dialog = gtk_message_dialog_new (NULL,
+		err_dialog = gtk_message_dialog_new (NULL,
 						 GTK_DIALOG_DESTROY_WITH_PARENT,
 						 GTK_MESSAGE_ERROR,
 						 GTK_BUTTONS_CLOSE,
 						 _("Unable to load"));
-		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (err_dialog),
 		   _("Cannot find some needed resources (the glade file)!"));
-		gtk_dialog_run (GTK_DIALOG (dialog));
-		gtk_widget_destroy (dialog);
+		gtk_dialog_run (GTK_DIALOG (err_dialog));
+		gtk_widget_destroy (err_dialog);
 
 		return FALSE;
 	}
@@ -993,18 +979,14 @@ init_app (void)
 	gtk_signal_connect (GTK_OBJECT (dialog), "delete_event",
 			    GTK_SIGNAL_FUNC (close_cb), NULL);
 
-
 	vpn_conn_view = GTK_TREE_VIEW (glade_xml_get_widget (xml, "vpnlist"));
 	vpn_conn_list = gtk_list_store_new (VPNCONN_N_COLUMNS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN);
 
 	gtk_signal_connect_after (GTK_OBJECT (vpn_conn_view), "cursor-changed",
 				  GTK_SIGNAL_FUNC (vpn_list_cursor_changed_cb), NULL);
 
-
 	get_all_vpn_connections ();
 
-	
-
 	column = gtk_tree_view_column_new ();
 	renderer = gtk_cell_renderer_text_new ();
 	gtk_tree_view_column_pack_start (column, renderer, TRUE);
@@ -1016,7 +998,6 @@ init_app (void)
 	gtk_tree_view_set_model (vpn_conn_view, GTK_TREE_MODEL (vpn_conn_list));
 	gtk_tree_view_expand_all (vpn_conn_view);
 
-       
 	gtk_widget_show_all (dialog);
 
 	/* fill in possibly choices in the druid when adding a connection */
@@ -1025,12 +1006,10 @@ init_app (void)
 	for (i = vpn_types; i != NULL; i = g_slist_next (i)) {
 		NetworkManagerVpnUI *vpn_ui = i->data;
 		gtk_combo_box_append_text (vpn_type_combo_box, vpn_ui->get_display_name (vpn_ui));
-
 	}
 	gtk_combo_box_set_active (vpn_type_combo_box, 0);
 	gtk_box_pack_end (GTK_BOX (vpn_type_hbox1), GTK_WIDGET (vpn_type_combo_box), TRUE, TRUE, 0);
 
-
 	/* Druid */
 	druid = GNOME_DRUID (glade_xml_get_widget (xml, "vpn-create-connection-druid"));
 	gtk_signal_connect (GTK_OBJECT (druid), "cancel", GTK_SIGNAL_FUNC (vpn_druid_cancel), NULL);
@@ -1046,14 +1025,11 @@ init_app (void)
 	gtk_signal_connect_after (GTK_OBJECT (w), "finish", GTK_SIGNAL_FUNC (vpn_druid_vpn_confirm_page_finish), NULL);
 
 	druid_window = GTK_WINDOW (glade_xml_get_widget (xml, "vpn-create-connection"));
-	/* reuse the widget */
-	gtk_signal_connect (GTK_OBJECT (druid_window), "delete-event", 
-			    GTK_SIGNAL_FUNC (gtk_widget_hide_on_delete), NULL);
+
 	/* make the druid window modal wrt. our main window */
 	gtk_window_set_modal (druid_window, TRUE);
 	gtk_window_set_transient_for (druid_window, GTK_WINDOW (dialog));
 
-
 	/* Edit dialog */
 	edit_dialog = GTK_DIALOG (gtk_dialog_new_with_buttons (_("Edit VPN Connection"),
 							       NULL,
@@ -1063,9 +1039,6 @@ init_app (void)
 							       GTK_STOCK_APPLY,
 							       GTK_RESPONSE_ACCEPT,
 							       NULL));
-	/* reuse the widget */
-	gtk_signal_connect (GTK_OBJECT (edit_dialog), "delete-event", 
-			    GTK_SIGNAL_FUNC (gtk_widget_hide_on_delete), NULL);
 
 	/* update "Edit" and "Delete" for current selection */
 	update_edit_del_sensitivity ();
@@ -1073,23 +1046,21 @@ init_app (void)
 	return TRUE;
 }
 
-
 int
 main (int argc, char *argv[])
 {
 	GOptionContext *context;
-	int ret = 1;
+	int ret;
 	gboolean bad_opts;
 	gboolean do_import;
 	GError *error = NULL;
-	static gchar *import_svc = NULL;
-	static gchar *import_file = NULL;
-	static GOptionEntry entries[] = 
-		{
-			{ "import-service", 's', 0,G_OPTION_ARG_STRING, &import_svc, "VPN Service for importing", NULL},
+	gchar *import_svc = NULL;
+	gchar *import_file = NULL;
+	GOptionEntry entries[] =  {
+			{ "import-service", 's', 0, G_OPTION_ARG_STRING, &import_svc, "VPN Service for importing", NULL},
 			{ "import-file", 'f', 0, G_OPTION_ARG_STRING, &import_file, "File to import", NULL},
-			{ NULL }
-		};
+			{ NULL, 0, 0, G_OPTION_ARG_NONE, NULL, NULL, NULL }
+	};
 
 	bindtextdomain (GETTEXT_PACKAGE, NULL);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
@@ -1103,39 +1074,35 @@ main (int argc, char *argv[])
 	bad_opts = FALSE;
 	do_import = FALSE;
 	if (import_svc != NULL) {
-		if (import_file != NULL) {
+		if (import_file != NULL)
 			do_import = TRUE;
-		} else {
+		else
 			bad_opts = TRUE;
-		}
-	} else {
-		if (import_file != NULL) {
+	} else if (import_file != NULL)
 			bad_opts = TRUE;
-		}
-	}
 
 	if (bad_opts) {
 		fprintf (stderr, "Have to supply both service and file\n");
+		ret = EXIT_FAILURE;
 		goto out;
 	}
 
 	gnome_program_init (GETTEXT_PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
-			    GNOME_PARAM_NONE);
+			    GNOME_PARAM_NONE, GNOME_PARAM_NONE);
 
 	glade_gnome_init ();
 
 	if (init_app () == FALSE) {
-		ret = 1;
+		ret = EXIT_FAILURE;
 		goto out;
 	}
 
-	if (do_import) {
-		import_settings (import_svc, import_file);		
-	}
+	if (do_import)
+		import_settings (import_svc, import_file);
 
 	gtk_main ();
 
-	ret = 0;
+	ret = EXIT_SUCCESS;
 
 out:
 	return ret;
