@@ -20,7 +20,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
  * (C) Copyright 2004 Red Hat, Inc.
- * (C) Copyright 2005 SuSE GmbH
+ * (C) Copyright 2005-2006 SuSE GmbH
  */
 
 #include <stdio.h>
@@ -31,7 +31,9 @@
 
 #include "NetworkManagerSystem.h"
 #include "NetworkManagerUtils.h"
-#include "NetworkManagerDevice.h"
+#include "nm-device.h"
+#include "nm-device-802-3-ethernet.h"
+#include "nm-device-802-11-wireless.h"
 #include "NetworkManagerDialup.h"
 #include "nm-utils.h"
 #include "shvar.h"
@@ -315,7 +317,10 @@ void nm_system_device_add_ip6_link_address (NMDevice *dev)
 	struct ether_addr hw_addr;
 	unsigned char eui[8];
 
-	nm_device_get_hw_address (dev, &hw_addr);
+	if (nm_device_is_802_3_ethernet (dev))
+		nm_device_802_3_ethernet_get_address (NM_DEVICE_802_3_ETHERNET (dev), &hw_addr);
+	else if (nm_device_is_802_11_wireless (dev))
+		nm_device_802_11_wireless_get_address (NM_DEVICE_802_11_WIRELESS (dev), &hw_addr);
 
 	memcpy (eui, &(hw_addr.ether_addr_octet), sizeof (hw_addr.ether_addr_octet));
 	memmove (eui+5, eui+3, 3);
@@ -332,11 +337,12 @@ void nm_system_device_add_ip6_link_address (NMDevice *dev)
 }
 
 
-typedef struct SuSESystemConfigData
+typedef struct SuSEDeviceConfigData
 {
 	NMIP4Config *	config;
 	gboolean		use_dhcp;
-} SuSESystemConfigData;
+	gboolean		system_disabled;
+} SuSEDeviceConfigData;
 
 /*
  * set_ip4_config_from_resolv_conf
@@ -429,27 +435,29 @@ void *nm_system_device_get_system_config (NMDevice *dev)
 	struct stat statbuf;
 	shvarFile *file;
 	char *buf = NULL;
-	SuSESystemConfigData *sys_data = NULL;
-	struct ether_addr addr;
+	SuSEDeviceConfigData *sys_data = NULL;
+	struct ether_addr hw_addr;
 	FILE *f = NULL;
 	char buffer[512];
 	gboolean error = FALSE;
-	unsigned int i;
-	int len;
+	int i, len;
 	struct in_addr temp_addr;
 	char *ip_str;
 
 	g_return_val_if_fail (dev != NULL, NULL);
 
-	sys_data = g_malloc0 (sizeof (SuSESystemConfigData));
+	sys_data = g_malloc0 (sizeof (SuSEDeviceConfigData));
 	sys_data->use_dhcp = TRUE;
 
-	memset (&addr, 0, sizeof(addr));
-	nm_device_get_hw_address (dev, &addr);
+	if (nm_device_is_802_3_ethernet (dev))
+		nm_device_802_3_ethernet_get_address (NM_DEVICE_802_3_ETHERNET (dev), &hw_addr);
+	else if (nm_device_is_802_11_wireless (dev))
+		nm_device_802_11_wireless_get_address (NM_DEVICE_802_11_WIRELESS (dev), &hw_addr);
+
 	sprintf (mac, "%02x:%02x:%02x:%02x:%02x:%02x",
-			addr.ether_addr_octet[0], addr.ether_addr_octet[1],
-			addr.ether_addr_octet[2], addr.ether_addr_octet[3],
-			addr.ether_addr_octet[4], addr.ether_addr_octet[5]);
+			hw_addr.ether_addr_octet[0], hw_addr.ether_addr_octet[1],
+			hw_addr.ether_addr_octet[2], hw_addr.ether_addr_octet[3],
+			hw_addr.ether_addr_octet[4], hw_addr.ether_addr_octet[5]);
 	cfg_file_path = g_strdup_printf (SYSCONFDIR"/sysconfig/network/ifcfg-eth-id-%s", mac);
 	if (!cfg_file_path)
 		return sys_data;
@@ -490,20 +498,41 @@ found:
 		free (buf);
 	}
 
+	if ((buf = svGetValue (file, "NM_CONTROLLED")))
+	{
+		nm_debug ("NM_CONTROLLED=%s", buf);
+		if (!strcasecmp (buf, "no"))
+		{
+			nm_info ("System configuration disables device %s", nm_device_get_iface (dev));
+			sys_data->system_disabled = TRUE;
+		}
+		free (buf);
+	}		
+
 	sys_data->config = nm_ip4_config_new ();
 
-	if (!(sys_data->use_dhcp))
+	if (!sys_data->use_dhcp || sys_data->system_disabled)
 	{
-		if ((buf = svGetValue (file, "IPADDR")))
+		buf = svGetValue (file, "IPADDR");
+		if (buf)
 		{
-			nm_ip4_config_set_address (sys_data->config, inet_addr (buf));
+			struct in_addr ip;
+			int ret;
+
+			ret = inet_aton (buf, &ip);
+			if (ret)
+				nm_ip4_config_set_address (sys_data->config, ip.s_addr);
+			else
+				error = TRUE;
 			free (buf);
 		}
 		else
-		{
-			nm_warning ("Network configuration for device '%s' was invalid (non-DHCP configuration, "
-						"but no IP address specified.  Will use DHCP instead.", nm_device_get_iface (dev));
 			error = TRUE;
+
+		if (error)
+		{
+			nm_warning ("Network configuration for device '%s' was invalid: Non-DHCP configuration, "
+					  "but no IP address specified.  Will use DHCP instead.", nm_device_get_iface (dev));
 			goto out;
 		}
 
@@ -554,12 +583,8 @@ found:
 			fclose (f);
 		}
 		if (!buf)
-		{
-			nm_warning ("Network configuration for device '%s' was invalid (non-DHCP configuration, "
-						"but no gateway specified.  Will use DHCP instead.", nm_device_get_iface (dev));
-			error = TRUE;
-			goto out;
-		}
+			nm_info ("Network configuration for device '%s' does not specify a gateway but is "
+				 "statically configured (non-DHCP).", nm_device_get_iface (dev));
 
 		set_ip4_config_from_resolv_conf (SYSCONFDIR"/resolv.conf", sys_data->config);
 	}
@@ -618,7 +643,7 @@ out:
  */
 void nm_system_device_free_system_config (NMDevice *dev, void *system_config_data)
 {
-	SuSESystemConfigData *sys_data = (SuSESystemConfigData *)system_config_data;
+	SuSEDeviceConfigData *sys_data = (SuSEDeviceConfigData *)system_config_data;
 
 	g_return_if_fail (dev != NULL);
 
@@ -639,20 +664,40 @@ void nm_system_device_free_system_config (NMDevice *dev, void *system_config_dat
  */
 gboolean nm_system_device_get_use_dhcp (NMDevice *dev)
 {
-	SuSESystemConfigData *sys_data;
+	SuSEDeviceConfigData *sys_data;
 
-	g_return_val_if_fail (dev != NULL, TRUE);
+	g_return_val_if_fail (dev != NULL, FALSE);
 
 	if ((sys_data = nm_device_get_system_config_data (dev)))
 		return sys_data->use_dhcp;
 
-	return TRUE;
+	return FALSE;
+}
+
+
+/*
+ * nm_system_device_get_disabled
+ *
+ * Return whether the distribution has flagged this device as disabled.
+ *
+ */
+gboolean nm_system_device_get_disabled (NMDevice *dev)
+{
+	SuSEDeviceConfigData *sys_data;
+
+	g_return_val_if_fail (dev != NULL, FALSE);
+
+
+	if ((sys_data = nm_device_get_system_config_data (dev)))
+		return sys_data->system_disabled;
+
+	return FALSE;
 }
 
 
 NMIP4Config *nm_system_device_new_ip4_system_config (NMDevice *dev)
 {
-	SuSESystemConfigData *sys_data;
+	SuSEDeviceConfigData *sys_data;
 	NMIP4Config *new_config = NULL;
 
 	g_return_val_if_fail (dev != NULL, NULL);
@@ -673,6 +718,13 @@ void nm_system_deactivate_all_dialup (GSList *list)
 		NMDialUpConfig *config = (NMDialUpConfig *) elt->data;
 		char *cmd;
 
+		if (config->type == NM_DIALUP_TYPE_ISDN)
+		{
+			cmd = g_strdup_printf ("/sbin/isdnctrl hangup %s", (char *) config->data);
+			nm_spawn_process (cmd);
+			g_free (cmd);
+		}
+
 		cmd = g_strdup_printf ("/sbin/ifdown %s", (char *) config->data);
 		nm_spawn_process (cmd);
 		g_free (cmd);
@@ -680,6 +732,40 @@ void nm_system_deactivate_all_dialup (GSList *list)
 }
 
 
+gboolean nm_system_deactivate_dialup (GSList *list, const char *dialup)
+{
+	GSList *elt;
+	gboolean ret = FALSE;
+
+	for (elt = list; elt; elt = g_slist_next (elt))
+	{
+		NMDialUpConfig *config = (NMDialUpConfig *) elt->data;
+		if (strcmp (dialup, config->name) == 0)
+		{
+			char *cmd;
+
+			nm_info ("Deactivating dialup device %s (%s) ...", dialup, (char *) config->data);
+
+			cmd = g_strdup_printf ("/sbin/ifdown %s", (char *) config->data);
+			nm_spawn_process (cmd);
+			g_free (cmd);
+
+			if (config->type == NM_DIALUP_TYPE_ISDN)
+			{
+				cmd = g_strdup_printf ("/sbin/isdnctrl hangup %s", (char *) config->data);
+				nm_spawn_process (cmd);
+				g_free (cmd);
+			}
+
+			ret = TRUE;
+			break;
+		}
+	}
+
+	return ret;
+}
+
+
 gboolean nm_system_activate_dialup (GSList *list, const char *dialup)
 {
 	GSList *elt;
@@ -693,9 +779,18 @@ gboolean nm_system_activate_dialup (GSList *list, const char *dialup)
 			char *cmd;
 
 			nm_info ("Activating dialup device %s (%s) ...", dialup, (char *) config->data);
+
 			cmd = g_strdup_printf ("/sbin/ifup %s", (char *) config->data);
 			nm_spawn_process (cmd);
 			g_free (cmd);
+
+			if (config->type == NM_DIALUP_TYPE_ISDN)
+			{
+				cmd = g_strdup_printf ("/sbin/isdnctrl dial %s", (char *) config->data);
+				nm_spawn_process (cmd);
+				g_free (cmd);
+			}
+
 			ret = TRUE;
 			break;
 		}
@@ -727,7 +822,8 @@ static char * verify_and_return_provider (const char *provider)
 		goto out_close;
 	ret = strcmp (buf, "no");
 	free (buf);
-	if (ret) {
+	if (ret)
+	{
 		buf = NULL;
 		goto out_close;
 	}
@@ -794,11 +890,16 @@ GSList * nm_system_get_dialup_config (void)
 			 goto out_free;
 
 		config = g_malloc (sizeof (NMDialUpConfig));
-		if (modem)
-			config->name = g_strdup_printf ("%s via Modem", provider_name);
-		else
-			config->name = g_strdup_printf ("%s via ISDN", provider_name);
 		config->data = g_strdup (dentry + 6); /* skip the "ifcfg-" prefix */
+		if (modem)
+		{
+			config->name = g_strdup_printf ("%s via modem (%s)", provider_name, (char *) config->data);
+			config->type = NM_DIALUP_TYPE_MODEM;
+		} else
+		{
+			config->name = g_strdup_printf ("%s via ISDN (%s)", provider_name, (char *) config->data);
+			config->type = NM_DIALUP_TYPE_ISDN;
+		}
 
 		list = g_slist_append (list, config);
 
@@ -817,3 +918,198 @@ out_gfree:
 
 	return list;
 }
+
+
+/*
+ * nm_system_activate_nis
+ *
+ * set up the nis domain and write a yp.conf
+ *
+ */
+void nm_system_activate_nis (NMIP4Config *config)
+{
+	shvarFile *file;
+	const char *nis_domain;
+	char *name, *buf = NULL; 
+	int num_nis_servers = 0; 
+	struct in_addr	temp_addr;
+	int i;
+	FILE *ypconf = NULL;
+
+	g_return_if_fail (config != NULL);
+
+	nis_domain = nm_ip4_config_get_nis_domain(config);
+
+	name = g_strdup_printf (SYSCONFDIR"/sysconfig/network/dhcp");
+	file = svNewFile (name);
+	if (!file)
+		goto out_gfree;
+
+	buf = svGetValue (file, "DHCLIENT_SET_DOMAINNAME");
+	if (!buf)
+		goto out_close;
+
+	if ((!strcmp (buf, "yes")) && nis_domain && (setdomainname (nis_domain, strlen (nis_domain)) < 0))
+			nm_warning ("Could not set nis domain name.");
+	free (buf);
+
+	buf = svGetValue (file, "DHCLIENT_MODIFY_NIS_CONF");
+	if (!buf)
+		goto out_close;
+
+	if (!strcmp (buf, "yes")) {
+		num_nis_servers = nm_ip4_config_get_num_nis_servers(config);
+		/* write out yp.conf and restart the daemon */
+		if (num_nis_servers > 0)
+		{
+			struct stat sb;
+
+			ypconf = fopen ("/etc/yp.conf", "w");
+
+			if (ypconf)
+			{
+				fprintf (ypconf, "# generated by NetworkManager, do not edit!\n\n");
+				for (i = 0; i < num_nis_servers; i++) {
+					temp_addr.s_addr = nm_ip4_config_get_nis_server (config, i);
+					fprintf (ypconf, "domain %s server %s\n", nis_domain, inet_ntoa (temp_addr));
+				}
+				fprintf (ypconf, "\n");
+				fclose (ypconf);
+			} else
+				nm_warning ("Could not commit NIS changes to /etc/yp.conf.");
+
+			if (stat ("/usr/sbin/rcypbind", &sb) != -1)
+			{
+				nm_info ("Restarting ypbind.");
+				nm_spawn_process ("/usr/sbin/rcypbind restart");
+			}
+			if (stat ("/usr/sbin/rcautofs", &sb) != -1)
+			{
+				nm_info ("Restarting autofs.");
+				nm_spawn_process ("/usr/sbin/rcautofs restart");
+			}
+		}
+	}
+	free (buf);
+
+out_close:
+	svCloseFile (file);
+out_gfree:
+	g_free (name);
+}
+
+
+/*
+ * nm_system_shutdown_nis
+ *
+ * shutdown ypbind
+ *
+ */
+void nm_system_shutdown_nis (void)
+{
+	struct stat sb;
+
+	if (stat ("/usr/sbin/rcypbind", &sb) != -1)
+	{
+		nm_info ("Stopping ypbind.");
+		nm_spawn_process ("/usr/sbin/rcypbind stop");
+	}
+	if (stat ("/usr/sbin/rcautofs", &sb) != -1)
+	{
+		nm_info ("Restarting autofs.");
+		nm_spawn_process ("/usr/sbin/rcautofs restart");
+	}
+}
+
+/*
+ * nm_system_set_hostname
+ *
+ * set the hostname
+ *
+ */
+void nm_system_set_hostname (NMIP4Config *config)
+{
+	char *filename, *h_name = NULL, *buf;
+	shvarFile *file;
+
+	g_return_if_fail (config != NULL);
+
+	filename = g_strdup_printf (SYSCONFDIR"/sysconfig/network/dhcp");
+	file = svNewFile (filename);
+	if (!file)
+		goto out_gfree;
+
+	buf = svGetValue (file, "DHCLIENT_SET_HOSTNAME");
+	if (!buf)
+		goto out_close;
+
+	if (!strcmp (buf, "yes")) 
+	{
+		const char *hostname;
+
+		hostname = nm_ip4_config_get_hostname (config);
+		if (!hostname)
+		{
+			struct in_addr temp_addr;
+			struct hostent *host;
+
+			/* try to get hostname via dns */
+			temp_addr.s_addr = nm_ip4_config_get_address (config);
+			host = gethostbyaddr ((char *) &temp_addr, sizeof (temp_addr), AF_INET);
+			if (host)
+			{
+				h_name = g_strdup (host->h_name);
+				hostname = strtok (h_name, ".");
+			}
+			else
+				nm_warning ("nm_system_set_hostname(): gethostbyaddr failed, h_errno = %d", h_errno);
+		}
+
+		if (hostname)
+		{
+			nm_info ("Setting hostname to '%s'", hostname);
+			if (sethostname (hostname, strlen (hostname)) < 0)
+				nm_warning ("Could not set hostname.");
+		}
+	}
+
+	g_free (h_name);
+	free (buf);
+out_close:
+	svCloseFile (file);
+out_gfree:
+	g_free (filename);
+}
+
+/*
+ * nm_system_should_modify_resolv_conf
+ *
+ * Can NM update resolv.conf, or is it locked down?
+ */
+gboolean nm_system_should_modify_resolv_conf (void)
+{
+	char *name, *buf;
+	shvarFile *file;
+	gboolean ret = TRUE;
+
+	name = g_strdup_printf (SYSCONFDIR"/sysconfig/network/config");
+	file = svNewFile (name);
+	if (!file)
+		goto out_gfree;
+
+	buf = svGetValue (file, "MODIFY_RESOLV_CONF_DYNAMICALLY");
+	if (!buf)
+		goto out_close;
+
+	if (strcmp (buf, "no") == 0)
+		ret = FALSE;
+
+	free (buf);
+out_close:
+	svCloseFile (file);
+out_gfree:
+	g_free (name);
+
+	return ret;
+}
+
