@@ -34,6 +34,9 @@
 #include "NetworkManagerDialup.h"
 #include "NetworkManagerSystem.h"
 #include "NetworkManager.h"
+#include "nm-ap-security.h"
+#include "nm-device-802-3-ethernet.h"
+#include "nm-device-802-11-wireless.h"
 
 
 /*
@@ -174,6 +177,39 @@ out:
 }
 
 
+static DBusMessage *nm_dbus_nm_deactivate_dialup (DBusConnection *connection, DBusMessage *message, NMDbusCBData *data)
+{
+	DBusMessage *reply = NULL;
+	NMData *nm_data = (NMData *) data->data;
+	const char *dialup;
+
+	g_return_val_if_fail (data != NULL, NULL);
+	g_return_val_if_fail (data->data != NULL, NULL);
+	g_return_val_if_fail (connection != NULL, NULL);
+	g_return_val_if_fail (message != NULL, NULL);
+
+	reply = dbus_message_new_method_return (message);
+	if (!reply)
+		return NULL;
+
+	if (!dbus_message_get_args (message, NULL, DBUS_TYPE_STRING, &dialup, DBUS_TYPE_INVALID))
+	{
+		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "InvalidArguments",
+									   "NetworkManager::deactivateDialup called with invalid arguments.");
+		goto out;
+	}
+
+	nm_lock_mutex (nm_data->dialup_list_mutex, __FUNCTION__);
+	if (!nm_system_deactivate_dialup (nm_data->dialup_list, dialup))
+		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "DeactivationFailed",
+									   "Failed to deactivate the dialup device.");
+	nm_unlock_mutex (nm_data->dialup_list_mutex, __FUNCTION__);
+
+out:
+	return reply;
+}
+
+
 /*
  * nm_dbus_nm_set_active_device
  *
@@ -183,62 +219,99 @@ out:
  */
 static DBusMessage *nm_dbus_nm_set_active_device (DBusConnection *connection, DBusMessage *message, NMDbusCBData *data)
 {
+	const char * INVALID_ARGS_ERROR = "InvalidArguments";
+	const char * INVALID_ARGS_MESSAGE = "NetworkManager::setActiveDevice called with invalid arguments.";
 	NMDevice *		dev = NULL;
 	DBusMessage *		reply = NULL;
-	char *			dev_path = NULL;
-	char *			essid = NULL;
-	char *			key = NULL;
-	int				key_type = -1;
+	char *			dev_path;
+	char *			unescaped_dev_path = NULL;
 	NMAccessPoint *	ap = NULL;
+	DBusMessageIter	iter;
 
 	g_return_val_if_fail (connection != NULL, NULL);
 	g_return_val_if_fail (message != NULL, NULL);
 	g_return_val_if_fail (data != NULL, NULL);
 	g_return_val_if_fail (data->data != NULL, NULL);
 
-	/* Try to grab both device _and_ network first, and if that fails then just the device. */
-	if (!dbus_message_get_args (message, NULL,	DBUS_TYPE_OBJECT_PATH, &dev_path,
-										DBUS_TYPE_STRING, &essid,
-										DBUS_TYPE_STRING, &key,
-										DBUS_TYPE_INT32, &key_type, DBUS_TYPE_INVALID))
-	{
-		/* So if that failed, try getting just the device */
-		if (!dbus_message_get_args (message, NULL, DBUS_TYPE_OBJECT_PATH, &dev_path, DBUS_TYPE_INVALID))
-		{
-			reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "InvalidArguments",
-							"NetworkManager::setActiveDevice called with invalid arguments.");
-			goto out;
-		} else nm_info ("FORCE: device '%s'", dev_path);
-	} else nm_info ("FORCE: device '%s', network '%s'", dev_path, essid);
+	dbus_message_iter_init (message, &iter);
 
-	/* So by now we have a valid device and possibly a network as well */
-	dev_path = nm_dbus_unescape_object_path (dev_path);
-	dev = nm_dbus_get_device_from_object_path (data->data, dev_path);
-	g_free (dev_path);
+	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_OBJECT_PATH)
+	{
+		nm_warning ("%s:%d (%s): Invalid arguments (first arg type was not OBJECT_PATH).",
+					__FILE__, __LINE__, __func__);
+		goto out;
+	}
+
+	dbus_message_iter_get_basic (&iter, &dev_path);
+	unescaped_dev_path = nm_dbus_unescape_object_path (dev_path);
+	dev = nm_dbus_get_device_from_object_path (data->data, unescaped_dev_path);
+	g_free (unescaped_dev_path);
+
+	/* Ensure the device exists in our list and is supported */
 	if (!dev || !(nm_device_get_capabilities (dev) & NM_DEVICE_CAP_NM_SUPPORTED))
 	{
 		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "DeviceNotFound",
 						"The requested network device does not exist.");
+		nm_warning ("%s:%d (%s): Invalid device (device not found).", __FILE__, __LINE__, __func__);
 		goto out;
 	}
 
-	/* Make sure network is valid and device is wireless */
-	if (nm_device_is_wireless (dev) && !essid)
+	if (nm_device_is_802_11_wireless (dev))
 	{
-		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "InvalidArguments",
-							"NetworkManager::setActiveDevice called with invalid arguments.");
-		goto out;
+		NMAPSecurity * 	security = NULL;
+		char *			essid = NULL;
+
+		if (!dbus_message_iter_next (&iter) || (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING))
+		{
+			nm_warning ("%s:%d (%s): Invalid argument type (essid).", __FILE__, __LINE__, __func__);
+			goto out;
+		}
+
+		/* grab ssid and ensure validity */
+		dbus_message_iter_get_basic (&iter, &essid);
+		if (!essid || (strlen (essid) <= 0))
+		{
+			nm_warning ("%s:%d (%s): Invalid argument (essid).", __FILE__, __LINE__, __func__);
+			goto out;
+		}
+
+		/* If there's security information, we use that.  If not, we
+		 * make some up from the scan list.
+		 */
+		if (dbus_message_iter_next (&iter))
+		{
+			if (!(security = nm_ap_security_new_deserialize (&iter)))
+			{
+				/* There was security info, but it was invalid */
+				reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, INVALID_ARGS_ERROR, INVALID_ARGS_MESSAGE);
+				nm_warning ("%s:%d (%s): Invalid argument (wireless security info).", __FILE__, __LINE__, __func__);
+				goto out;
+			}
+		}
+
+		/* Set up the wireless-specific activation request properties */
+		ap = nm_device_802_11_wireless_get_activation_ap (NM_DEVICE_802_11_WIRELESS (dev), essid, security);
+		if (security)
+	 		g_object_unref (G_OBJECT (security));
+
+		nm_info ("User Switch: %s / %s", dev_path, essid);
+	}
+	else if (nm_device_is_802_3_ethernet (dev))
+	{
+		nm_info ("User Switch: %s", dev_path);
 	}
 
 	nm_device_deactivate (dev);
-
 	nm_schedule_state_change_signal_broadcast (data->data);
-
-	if (nm_device_is_wireless (dev))
-		ap = nm_device_wireless_get_activation_ap (dev, essid, key, (NMEncKeyType)key_type);
 	nm_policy_schedule_device_activation (nm_act_request_new (data->data, dev, ap, TRUE));
 
 out:
+	if (!reply)
+	{
+		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE,
+						INVALID_ARGS_ERROR, INVALID_ARGS_MESSAGE);
+	}
+
 	return reply;
 }
 
@@ -250,68 +323,77 @@ out:
  */
 static DBusMessage *nm_dbus_nm_create_wireless_network (DBusConnection *connection, DBusMessage *message, NMDbusCBData *data)
 {
+	const char *		INVALID_ARGS_ERROR = "InvalidArguments";
+	const char *		INVALID_ARGS_MESSAGE = "NetworkManager::createWirelessNetwork called with invalid arguments.";
 	NMDevice *		dev = NULL;
 	DBusMessage *		reply = NULL;
 	char *			dev_path = NULL;
+	char *			unescaped_dev_path = NULL;
 	NMAccessPoint *	new_ap = NULL;
-	char *			network = NULL;
-	char *			key = NULL;
-	int				key_type = -1;
-	DBusError			error;
+	NMAPSecurity * 	security = NULL;
+	char *			essid = NULL;
+	DBusMessageIter	iter;
 
 	g_return_val_if_fail (connection != NULL, NULL);
 	g_return_val_if_fail (message != NULL, NULL);
 	g_return_val_if_fail (data != NULL, NULL);
 	g_return_val_if_fail (data->data != NULL, NULL);
 
-	/* Try to grab both device _and_ network first, and if that fails then just the device. */
-	dbus_error_init (&error);
-	if (!dbus_message_get_args (message, &error, DBUS_TYPE_OBJECT_PATH, &dev_path,
-										DBUS_TYPE_STRING, &network,
-										DBUS_TYPE_STRING, &key,
-										DBUS_TYPE_INT32, &key_type, DBUS_TYPE_INVALID))
-	{
-		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "InvalidArguments",
-						"NetworkManager::createWirelessNetwork called with invalid arguments.");
-		return reply;
-	} else nm_info ("Creating network '%s' on device '%s'.", network, dev_path);
+	dbus_message_iter_init (message, &iter);
 
-	dev_path = nm_dbus_unescape_object_path (dev_path);
-	dev = nm_dbus_get_device_from_object_path (data->data, dev_path);
-	g_free (dev_path);
+	if (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_OBJECT_PATH)
+	{
+		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, INVALID_ARGS_ERROR, INVALID_ARGS_MESSAGE);
+		goto out;
+	}
+
+	dbus_message_iter_get_basic (&iter, &dev_path);
+	unescaped_dev_path = nm_dbus_unescape_object_path (dev_path);
+	dev = nm_dbus_get_device_from_object_path (data->data, unescaped_dev_path);
+	g_free (unescaped_dev_path);
+
+	/* Ensure the device exists in our list and is supported */
 	if (!dev || !(nm_device_get_capabilities (dev) & NM_DEVICE_CAP_NM_SUPPORTED))
 	{
 		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "DeviceNotFound",
 						"The requested network device does not exist.");
-		return reply;
-	}
-	nm_device_ref (dev);
-
-	/* Make sure network is valid and device is wireless */
-	if (!nm_device_is_wireless (dev) || !network)
-	{
-		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, "InvalidArguments",
-							"NetworkManager::createWirelessNetwork called with invalid arguments.");
 		goto out;
 	}
 
-	new_ap = nm_ap_new ();
-
-	/* Fill in the description of the network to create */
-	nm_ap_set_essid (new_ap, network);
-	if (nm_is_enc_key_valid (key, (NMEncKeyType)key_type))
+	if (    !nm_device_is_802_11_wireless (dev)
+		|| !dbus_message_iter_next (&iter)
+		|| (dbus_message_iter_get_arg_type (&iter) != DBUS_TYPE_STRING))
 	{
-		nm_ap_set_encrypted (new_ap, TRUE);
-		nm_ap_set_enc_key_source (new_ap, key, (NMEncKeyType)key_type);
-		nm_ap_set_auth_method (new_ap, NM_DEVICE_AUTH_METHOD_OPEN_SYSTEM);
+		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, INVALID_ARGS_ERROR, INVALID_ARGS_MESSAGE);
+		goto out;
 	}
-	nm_ap_set_mode (new_ap, NETWORK_MODE_ADHOC);
+
+	/* grab ssid and ensure validity */
+	dbus_message_iter_get_basic (&iter, &essid);
+	if (!essid || (strlen (essid) <= 0) || !dbus_message_iter_next (&iter))
+	{
+		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, INVALID_ARGS_ERROR, INVALID_ARGS_MESSAGE);
+		goto out;
+	}
+
+	if (!(security = nm_ap_security_new_deserialize (&iter)))
+	{
+		reply = nm_dbus_create_error_message (message, NM_DBUS_INTERFACE, INVALID_ARGS_ERROR, INVALID_ARGS_MESSAGE);
+		goto out;
+	}
+
+	nm_info ("Creating network '%s' on device '%s'.", essid, dev_path);
+
+	new_ap = nm_ap_new ();
+	nm_ap_set_essid (new_ap, essid);
+	nm_ap_set_mode (new_ap, IW_MODE_ADHOC);
+	nm_ap_set_security (new_ap, security);
+	g_object_unref (G_OBJECT (security));
 	nm_ap_set_user_created (new_ap, TRUE);
 
 	nm_policy_schedule_device_activation (nm_act_request_new (data->data, dev, new_ap, TRUE));
 
 out:
-	nm_device_unref (dev);
 	return reply;
 }
 
@@ -327,7 +409,7 @@ static DBusMessage *nm_dbus_nm_create_test_device (DBusConnection *connection, D
 
 	dbus_error_init (&err);
 	if (    dbus_message_get_args (message, &err, DBUS_TYPE_INT32, &type, DBUS_TYPE_INVALID)
-		&& ((type == DEVICE_TYPE_WIRED_ETHERNET) || (type == DEVICE_TYPE_WIRELESS_ETHERNET)))
+		&& ((type == DEVICE_TYPE_802_3_ETHERNET) || (type == DEVICE_TYPE_802_11_WIRELESS)))
 	{
 		char			*interface = g_strdup_printf ("test%d", test_dev_num);
 		char			*udi = g_strdup_printf ("/test-devices/%s", interface);
@@ -420,7 +502,7 @@ static DBusMessage *nm_dbus_nm_set_wireless_enabled (DBusConnection *connection,
 		for (elt = app_data->dev_list; elt; elt = g_slist_next (elt))
 		{
 			NMDevice	*dev = (NMDevice *)(elt->data);
-			if (nm_device_is_wireless (dev))
+			if (nm_device_is_802_11_wireless (dev))
 			{
 				nm_device_deactivate (dev);
 				nm_device_bring_down (dev);
@@ -472,6 +554,7 @@ static DBusMessage *nm_dbus_nm_sleep (DBusConnection *connection, DBusMessage *m
 			NMDevice *dev = (NMDevice *)(elt->data);
 			nm_device_set_removed (dev, TRUE);
 			nm_device_deactivate_quickly (dev);
+			nm_system_device_set_up_down (dev, FALSE);
 		}
 		nm_unlock_mutex (app_data->dev_list_mutex, __FUNCTION__);
 
@@ -538,6 +621,7 @@ NMDbusMethodList *nm_dbus_nm_methods_setup (void)
 	nm_dbus_method_list_add_method (list, "getDevices",			nm_dbus_nm_get_devices);
 	nm_dbus_method_list_add_method (list, "getDialup",			nm_dbus_nm_get_dialup);
 	nm_dbus_method_list_add_method (list, "activateDialup",		nm_dbus_nm_activate_dialup);
+	nm_dbus_method_list_add_method (list, "deactivateDialup",		nm_dbus_nm_deactivate_dialup);
 	nm_dbus_method_list_add_method (list, "setActiveDevice",		nm_dbus_nm_set_active_device);
 	nm_dbus_method_list_add_method (list, "createWirelessNetwork",	nm_dbus_nm_create_wireless_network);
 	nm_dbus_method_list_add_method (list, "setWirelessEnabled",		nm_dbus_nm_set_wireless_enabled);
